#ifndef __anim_format__H
#define __anim_format__H

#include <windows.h>

/////////////////////////////////////////////////////////////////////////////////////////////
#pragma pack(push, 1)

#ifndef TimeValue
	typedef int TimeValue;
#endif

#define MAX_ANIMNAME MAX_PATH // massima lunghezza per il nome di una animazione

typedef struct {
	unsigned int	StreamSize; // questo DEVE essere il primo membro!!!
	char			AnimName [MAX_ANIMNAME];
	char			ClientName [MAX_COMPUTERNAME_LENGTH + 1];
} anim_stream_desc_t;

typedef struct {
	int				first_frame;
	int				last_frame;
	int				frame_speed;
	int				ticks_per_frame;
} global_info_t;

typedef unsigned short tag_type;

#define TAG_GROUP	0
typedef struct {
	tag_type			tag; // = TAG_GROUP
	int					node_id;
	int					parent_node_id; // 0 = scene root
	int					node_name_lenght;
		// seguono node_name_lenght caratteri che rappresentano il nome del nodo
} scene_group_t;

typedef struct {
	float m[4][3];
} AnimMatrix3_t;

typedef struct {
	float x, y, z;
} AnimPoint3_t;

typedef struct {
	float x,y,z,w;
} AnimQuat_t;

typedef struct {
	AnimPoint3_t t;
	AnimQuat_t q;
	AnimQuat_t u;
	AnimPoint3_t k;
	float f;
} AnimAffineParts_t;

#define TAG_GEOMOBJ	1
typedef struct {
	tag_type			tag; // = TAG_GEOMOBJ
	unsigned short		node_type;
#define NODETYPE_BIPEDBONE				0
#define NODETYPE_RIGID					1
#define NODETYPE_DEFORMATION			2
#define NODETYPE_PHYSIQUE				3
	int					node_id;
	int					parent_node_id; // 0 = scene root
	int					node_name_lenght;
	unsigned char		inherit_pos[3];
	unsigned char		inherit_rot[3];
	unsigned char		inherit_scl[3];
	AnimAffineParts_t	affine_tm;
	AnimMatrix3_t		figure_tm; // se node_type==BIPEDBONE altrimenti  identity...
		// seguono node_name_lenght caratteri che rappresentano il nome del nodo
} scene_geomobj_t;

#define TAG_KEYS	2
typedef struct {
	tag_type			tag; // = TAG_KEYS
	int					position_keys;
	int					position_controller;
#define POSITIONCONTROLLER_TCB			0
#define POSITIONCONTROLLER_HYBRID		1
#define POSITIONCONTROLLER_LIN			2
	int					rotation_keys;
	int					rotation_controller;
#define ROTATIONCONTROLLER_TCB			0
#define ROTATIONCONTROLLER_HYBRID		1
#define ROTATIONCONTROLLER_LIN			2
	int					scale_keys;
	int					scale_controller;
#define SCALECONTROLLER_TCB				0
#define SCALECONTROLLER_HYBRID			1
#define SCALECONTROLLER_LIN				2
		// seguono i dettagli su ciascuna key nell'ordine: position, rotation e scale.
} scene_keys_t;

#define TAG_SAMPTM	3
typedef struct {
	tag_type			tag; // = TAG_SAMPTM
	int					keys_num;
		// seguono keys_num TMSample_t
} scene_samptm_t;

#define TAG_MESH	4
typedef struct {
	tag_type			tag; // = TAG_MESH
	TimeValue			time;
	int					num_verts;
	int					num_faces;
	int					num_tverts;
	unsigned char		has_normals;
	AnimMatrix3_t		tm;
		// seguono:			num_verts AnimPoint3_t
		//					num_faces AnimFace_t
		//					se has_normals != 0:	num_faces AnimFaceNormal_t
		//					se num_tverts != 0:		num_tverts AnimUVVert_t
		//											num_faces AnimTVFace_t
} scene_mesh_t;

#define TAG_MATLIST	5
typedef struct {
	tag_type			tag; // = TAG_MATLIST
	int					mat_count;
		// seguono mat_count scene_material_t
} scene_matlist_t;

typedef struct {
	int					material_id;
	int					name_lenght;
		// seguono name_lenght caratteri che rappresentano il nome del materiale
} scene_material_t;

#define TAG_PHYSVERTS	6
typedef struct {
	tag_type			tag; // = TAG_PHYSVERTS
	int					num_verts;
		// seguono num_verts scene_rigidvert_t o scene_blendedvert_t
} scene_physverts_t;

#define TAG_RIGIDVERT	7
typedef struct {
	tag_type			tag; // = TAG_RIGIDVERT
	int					bone_id;
} scene_rigidvert_t;

#define TAG_BLENDEDVERT	8
typedef struct {
	tag_type			tag; // = TAG_BLENDEDVERT
	int					bone_num;
		// seguono bone_num scene_boneweight_t
} scene_blendedvert_t;

typedef struct {
	int					bone_id;
	float				bone_weight;
} scene_boneweight_t;

typedef struct {
	DWORD v[3];
	DWORD smGroup;
	int material_id;
} AnimFace_t;

typedef struct {
	DWORD t[3];
} AnimTVFace_t;

typedef AnimPoint3_t AnimUVVert_t;

typedef struct {
	AnimPoint3_t			face_normal;
	AnimPoint3_t			vertex_normals[3];
} AnimFaceNormal_t;

typedef struct {
	AnimPoint3_t axis;
	float angle;
} AnimAngAxis_t;

typedef struct {
	AnimPoint3_t s;
	AnimQuat_t q;
} AnimScaleValue_t;

typedef struct {
	TimeValue			time;
	AnimAffineParts_t	tm;
} TMSample_t;

typedef struct {
	TimeValue			time;
	AnimPoint3_t		val;
	float				tens;
	float				cont;
	float				bias;
	float				easeIn;
	float				easeOut;
} TCB_POSkey_t;

typedef struct {
	TimeValue			time;
	AnimPoint3_t		val;
	AnimPoint3_t		intan;
	AnimPoint3_t		outtan;
	DWORD				flags;
} HYB_POSkey_t;

typedef struct {
	TimeValue			time;
	AnimPoint3_t		val;
} LIN_POSkey_t;

typedef struct {
	TimeValue			time;
	AnimAngAxis_t		val;
	float				tens;
	float				cont;
	float				bias;
	float				easeIn;
	float				easeOut;
	AnimQuat_t			key;
} TCB_ROTkey_t;

typedef struct {
	TimeValue			time;
	AnimQuat_t			val;
	AnimQuat_t			key;
} HYB_ROTkey_t;

typedef struct {
	TimeValue			time;
	AnimQuat_t			val;
	AnimQuat_t			key;
} LIN_ROTkey_t;

typedef struct {
	TimeValue			time;
	AnimScaleValue_t	val;
	float				tens;
	float				cont;
	float				bias;
	float				easeIn;
	float				easeOut;
} TCB_SCLkey_t;

typedef struct {
	TimeValue			time;
	AnimScaleValue_t	val;
	AnimPoint3_t		intan;
	AnimPoint3_t		outtan;
	DWORD				flags;
} HYB_SCLkey_t;

typedef struct {
	TimeValue			time;
	AnimScaleValue_t	val;
} LIN_SCLkey_t;

#pragma pack(pop)
/////////////////////////////////////////////////////////////////////////////////////////////

#endif