#include "MathLib.h"
#pragma warning (disable:4244)

//====================
//rad
//====================
BOOL rad=FALSE;

//====================
//IdentityMatrix
//
//Definizione della matrice identit
//====================
matrix_t IdentityMatrix = {
{	
	{ 1.0, 0.0, 0.0 },
	{ 0.0, 1.0, 0.0 },
	{ 0.0, 0.0, 1.0 }
},
	{ 0.0, 0.0, 0.0 }
};

//========================
//MatrixGenerateTranslation
//
//Genera una matrice di traslazione
//========================
void MatrixGenerateTranslation (matrix_t *m, vector_t *vec)
{
	// rende la matrice m una matrice identit
	*m=IdentityMatrix;

	// imposta il vettore della traslazione
	m->t[0]=vec->x;
	m->t[1]=vec->y;
	m->t[2]=vec->z;

	// ritorna
	return;
}

//=======================
//MatrixGenerateScaling
//
//Genera una matrice per scalare un punto
//=======================
void MatrixGenerateScaling (matrix_t *m, vector_t *vec)
{
	// rende la matrice m una matrice identit
	*m=IdentityMatrix;

	// imposta la matrice
	m->v[0][0]=vec->x;
	m->v[1][1]=vec->y;
	m->v[2][2]=vec->z;

	// ritorna
	return;
}

//=======================
//MatrixGenerateXRotate
//
//Genera una matrice di rotazione attorno all'asse x
//=======================
void MatrixGenerateXRotate (matrix_t *m, float r)
{
	float c=(float)MathLibCos(r);
	float s=(float)MathLibSin(r);

	// rende la matrice m una matrice identit
	*m=IdentityMatrix;

	// imposta gli elementi della matrice
	m->v[1][1]=c;
	m->v[2][1]=-s;
	m->v[1][2]=s;
	m->v[2][2]=c;

	// ritorna
	return;
}

//=======================
//MatrixGenerateYRotate
//
//Genera una matrice di rotazione attorno all'asse y
//=======================
void MatrixGenerateYRotate (matrix_t *m, float r)
{
	float c=(float)MathLibCos(r);
	float s=(float)MathLibSin(r);

	// rende la matrice m una matrice identit
	*m=IdentityMatrix;

	// imposta gli elementi della matrice
	m->v[0][0]=c;
	m->v[2][0]=s;
	m->v[0][2]=-s;
	m->v[2][2]=c;

	// ritorna
	return;
}

//=======================
//MatrixGenerateZRotate
//
//Genera una matrice di rotazione attorno all'asse z
//=======================
void MatrixGenerateZRotate (matrix_t *m, float r)
{
	float c=(float)MathLibCos(r);
	float s=(float)MathLibSin(r);

	// rende la matrice m una matrice identit
	*m=IdentityMatrix;

	// imposta gli elementi della matrice
	m->v[0][0]=c;
	m->v[1][0]=-s;
	m->v[0][1]=s;
	m->v[1][1]=c;

	// ritorna
	return;
}

//=======================
//MatrixGenerateRotation
//
//Genera una matrice di rotazione
//=======================
void MatrixGenerateRotation (matrix_t *m, float x, float y, float z)
{
	float sin_x=(float)MathLibSin(x);
	float cos_x=(float)MathLibCos(x);
	float sin_y=(float)MathLibSin(y);
	float cos_y=(float)MathLibCos(y);
	float sin_z=(float)MathLibSin(z);
	float cos_z=(float)MathLibCos(z);
	float sinx_siny=sin_x*sin_y;
	float cosx_siny=cos_x*sin_y;

	// imposta gli elementi della matrice
	m->v[0][0]=cos_y*cos_z;
	m->v[1][0]=cos_y*sin_z;
	m->v[2][0]=-sin_y;
	m->v[0][1]=(sinx_siny*cos_z)-(cos_x*sin_z);
	m->v[1][1]=(sinx_siny*sin_z)+(cos_x*cos_z);
	m->v[2][1]=sin_x*cos_y;
	m->v[0][2]=(cosx_siny*cos_z)+(sin_x*sin_z);
	m->v[1][2]=(cosx_siny*sin_z)-(sin_x*cos_z);
	m->v[2][2]=cos_x*cos_y;
	m->t[0]=0;
	m->t[1]=0;
	m->t[2]=0;

	// ritorna
	return;
}

//=======================
//MatrixGenerateVectorRotation
//
//Genera una matrice di rotazione attorno ad un vettore
//=======================
void MatrixGenerateVectorRotation (matrix_t *m, vector_t *vec, float a)
{
	float c=(float)MathLibCos(a);
	float s=(float)MathLibSin(a);
	float cc=1-c;
	vector_t vector=*vec;

	// normalizza il vettore
	VectorNormalize(&vector);

	// imposta gli elementi della matrice
	m->v[0][0]=(cc*vector.x*vector.x)+c;
	m->v[1][0]=(cc*vector.x*vector.y)+(vector.z*s);
	m->v[2][0]=(cc*vector.x*vector.z)-(vector.y*s);
	m->v[0][1]=(cc*vector.x*vector.y)-(vector.z*s);
	m->v[1][1]=(cc*vector.y*vector.y)+c;
	m->v[2][1]=(cc*vector.z*vector.y)+(vector.x*s);
	m->v[0][2]=(cc*vector.x*vector.z)+(vector.y*s);
	m->v[1][2]=(cc*vector.y*vector.z)-(vector.x*s);
	m->v[2][2]=(cc*vector.z*vector.z)+c;
	m->t[0]=0;
	m->t[1]=0;
	m->t[2]=0;

	// ritorna
	return;
}

//=======================
//MatrixMultiply
//
//Moltiplica due matrici
//=======================
void MatrixMultiply (matrix_t *m1, matrix_t *m2, matrix_t *out)
{
	matrix_t temp;
	int i,j;

	// se una delle due matrici m1 ed m2 coincide con out, esegue una copia
	if (m1==out) {
		temp=*m1;
		m1=&temp;
	}
	else if (m2==out) {
		temp=*m2;
		m2=&temp;
	}

	// esegue la moltiplicazione
	for (i=0;i<3;i++) {
		for (j=0;j<3;j++) {
			out->v[i][j]=(m1->v[i][0]*m2->v[0][j])+
				(m1->v[i][1]*m2->v[1][j])+
				(m1->v[i][2]*m2->v[2][j]);
		}
		out->t[i]=(m1->t[0]*m2->v[0][i])+
			(m1->t[1]*m2->v[1][i])+
			(m1->t[2]*m2->v[2][i])+
			m2->t[i];
	}

	// ritorna
	return;
}

//=======================
//VectorLenght
//
//Restituisce la lunghezza di un vettore
//=======================
float VectorLenght (vector_t *vec)
{
	// ritorna
	return((float)sqrt(vec->x*vec->x+vec->y*vec->y+vec->z*vec->z));
}

//=======================
//VectorNormalize
//
//Normalizza un vettore
//=======================
void VectorNormalize (vector_t *vec)
{
	float length=1.0f/VectorLenght(vec);

	// normalizza il vettore
	vec->x*=length;
	vec->y*=length;
	vec->z*=length;

	// ritorna
	return;
}

//=======================
//VectorCrossP
//
//Calcola il prodotto vettoriale
//=======================
void VectorCrossP (vector_t *vec1, vector_t *vec2, vector_t *out)
{
	// esegue il prodotto vettoriale
	out->x=(vec1->y*vec2->z)-(vec1->z*vec2->y);
	out->y=(vec1->z*vec2->x)-(vec1->x*vec2->z);
	out->z=(vec1->x*vec2->y)-(vec1->y*vec2->x);

	// ritorna
	return;
}

//=======================
//VectorDotP
//
//Restituisce il prodotto scalare
//=======================
float VectorDotP (vector_t *vec1, vector_t *vec2)
{
	// ritorna
	return((vec1->x*vec2->x)+(vec1->y*vec2->y)+(vec1->z*vec2->z));
}

//=======================
//MatrixApply
//
//Applica una matrice ad un punto
//=======================
void MatrixApply (matrix_t *m, point_t *p_in, point_t *p_out)
{
	// imposta gli elementi della matrice
	p_out->x=p_in->x*m->v[0][0]+p_in->y*m->v[1][0]+p_in->z*m->v[2][0]+m->t[0];
	p_out->y=p_in->x*m->v[0][1]+p_in->y*m->v[1][1]+p_in->z*m->v[2][1]+m->t[1];
	p_out->z=p_in->x*m->v[0][2]+p_in->y*m->v[1][2]+p_in->z*m->v[2][2]+m->t[2];

	// ritorna
	return;
}

//=========================
//VectorCopy
//
//Copia un vettore
//=========================
void VectorCopy (vector_t *src, vector_t *dest)
{
	// esegue l'operazione
	*dest=*src;

	// ritorna
	return;
}

//========================
//VectorAdd
//
//Addiziona due vettori
//========================
void VectorAdd (vector_t *vec0, vector_t *vec1, vector_t *dest)
{
	// esegue l'operazione
	dest->x=vec0->x+vec1->x;
	dest->y=vec0->y+vec1->y;	
	dest->z=vec0->z+vec1->z;

	// ritorna
	return;
}

//==========================
//VectorSubtract
//
//Sottrae un vettore ad un altro
//==========================
void VectorSubtract (vector_t *vec0, vector_t *vec1, vector_t *dest)
{
	// esegue l'operazione
	dest->x=vec0->x-vec1->x;
	dest->y=vec0->y-vec1->y;
	dest->z=vec0->z-vec1->z;

	// ritorna
	return;
}

//========================
//VectorMultByScalar
//
//Moltiplica un vettore per uno scalare
//========================
void VectorMultByScalar (vector_t *vec, float scalar, vector_t *dest)
{
	// esegue l'operazione
	dest->x=vec->x*scalar;
	dest->y=vec->y*scalar;
	dest->z=vec->z*scalar;

	// ritorna
	return;
}

//=======================
//origin
//=======================
point_t origin = {0,0,0};

//=======================
//QuatMul
//=======================
void QuatMul(quat_t *q1, quat_t *q2, quat_t *res)
{
	float A, B, C, D, E, F, G, H;

	A = (q1->w + q1->x)*(q2->w + q2->x);
	B = (q1->z - q1->y)*(q2->y - q2->z);
	C = (q1->x - q1->w)*(q2->y + q2->z);
	D = (q1->y + q1->z)*(q2->x - q2->w);
	E = (q1->x + q1->z)*(q2->x + q2->y);
	F = (q1->x - q1->z)*(q2->x - q2->y);
	G = (q1->w + q1->y)*(q2->w - q2->z);
	H = (q1->w - q1->y)*(q2->w + q2->z);

	res->w =  B + (-E - F + G + H) * 0.5f;
	res->x =  A - (E + F + G + H)  * 0.5f; 
	res->y = -C + (E - F + G - H)  * 0.5f;
	res->z = - D + (E - F - G + H) * 0.5f;
}

//=======================
//QuatSlerp
//=======================
#define DELTA 1e-6
void QuatSlerp(quat_t *from, quat_t *to, float t, quat_t *res)
{
	quat_t to1;
	double omega, cosom, sinom, scale0, scale1;

	cosom = from->x * to->x + from->y * to->y + from->z * to->z + from->w * to->w;

	if(cosom < 0.0)
	{
		cosom = -cosom;
		to1.x = - to->x;
		to1.y = - to->y;
		to1.z = - to->z;
		to1.w = - to->w;
	}
	else
	{
		to1.x = to->x;
		to1.y = to->y;
		to1.z = to->z;
		to1.w = to->w;
	}

	if((1.0 - cosom) > DELTA)
	{
		omega = acos(cosom);
		sinom = sin(omega);
		scale0 = sin((1.0 - t) * omega) / sinom;
		scale1 = sin(t * omega) / sinom;
	}
	else
	{        
		scale0 = 1.0 - t;
		scale1 = t;
	}

	res->x = scale0 * from->x + scale1 * to1.x;
	res->y = scale0 * from->y + scale1 * to1.y;
	res->z = scale0 * from->z + scale1 * to1.z;
	res->w = scale0 * from->w + scale1 * to1.w;
}
#undef DELTA

//=======================
//QuatToMatrix
//=======================
void QuatToMatrix(quat_t *quat, matrix_t *m)
{
	float wx, wy, wz, xx, yy, yz, xy, xz, zz, x2, y2, z2;

	x2 = quat->x + quat->x;
	y2 = quat->y + quat->y; 
	z2 = quat->z + quat->z;

	xx = quat->x * x2;
	xy = quat->x * y2;
	xz = quat->x * z2;

	yy = quat->y * y2;
	yz = quat->y * z2;
	zz = quat->z * z2;

	wx = quat->w * x2;
	wy = quat->w * y2;
	wz = quat->w * z2;

	m->v[0][0] = 1.0 - (yy + zz);
	m->v[0][1] = xy - wz;
	m->v[0][2] = xz + wy;

	m->v[1][0] = xy + wz;
	m->v[1][1] = 1.0 - (xx + zz);
	m->v[1][2] = yz - wx;

	m->v[2][0] = xz - wy;
	m->v[2][1] = yz + wx;
	m->v[2][2] = 1.0 - (xx + yy);

	m->t[0] = 0;
	m->t[1] = 0;
	m->t[2] = 0;
}

//=======================
//QuatToAngleAxis
//=======================
void QuatToAngleAxis(quat_t *quat, angleaxis_t *angleaxis)
{
	float scale, tw;
	tw = (float)acos(quat->w) * 2.f;
	scale = (float)sin(tw * 0.5);
	angleaxis->axis.x = quat->x / scale;
	angleaxis->axis.y = quat->y / scale;
	angleaxis->axis.z = quat->z / scale;
	angleaxis->angle = (tw * 180.f) / (float)PI;
}

//=======================
//QuatFromEuler
//=======================
void QuatFromEuler(float roll, float pitch, float yaw, quat_t *quat)
{
	float cr, cp, cy, sr, sp, sy, cpcy, spsy;

	cr = cos(roll * 0.5f);
	cp = cos(pitch * 0.5f);
	cy = cos(yaw * 0.5f);

	sr = sin(roll * 0.5f);
	sp = sin(pitch * 0.5f);
	sy = sin(yaw * 0.5f);

	cpcy = cp * cy;
	spsy = sp * sy;

	quat->w = cr * cpcy + sr * spsy;
	quat->x = sr * cpcy - cr * spsy;
	quat->y = cr * sp * cy + sr * cp * sy;
	quat->z = cr * cp * sy - sr * sp * cy;
}

//=======================
//MatrixFromAnimMatrix3
//=======================
void MatrixFromAnimMatrix3 (AnimMatrix3_t *in, matrix_t *out)
{
	for ( int i=0; i<3; i++ ) {
		for ( int j=0; j<3; j++ )
			out->v[i][j] = in->m[i][j];
		out->t[i] = in->m[3][i];
	}
}

//===================
//PlaneSetType
//
//Imposta il tipo di piano
//===================
void PlaneSetType (plane_t *p)
{
	float x,y,z;

	// calcola i valori assoluti delle componenti della normale
	x=(float)fabs ( p->normal.x );
	y=(float)fabs ( p->normal.y );
	z=(float)fabs ( p->normal.z );

	// stabilisce il tipo di piano
	if (x>=y && x>=z)
		p->type=0;
	else if (y>=x && y>=z)
		p->type=1;
	else
		p->type=2;

	// ritorna
	return;
}

//==========================
//PlaneInvertNormal
//
//Inverte la normale di un piano
//==========================
void PlaneInvertNormal (plane_t *p)
{
	// inverte la normale
	VectorSubtract(&origin,&p->normal,&p->normal);
	p->dist=-p->dist;

	// ritorna
	return;
}

//==========================
//VectorType
//
//Restituisce il tipo di vettore
//==========================
int VectorType (vector_t *v)
{
	float x,y,z;

	// esegue l'operazione
	x=(float)fabs ( v->x );
	y=(float)fabs ( v->y );
	z=(float)fabs ( v->z );
	if (x>=y && x>=z)
		return(0);
	else if (y>=x && y>=z)
		return(1);
	else
		return(2);
}