#ifndef __MATHLIB_H__
#define __MATHLIB_H__

#include <windows.h>
#include <math.h>

#include "Include\anim_format.h"

//===========================================================================================

typedef AnimPoint3_t point_t;
typedef AnimPoint3_t vector_t;
typedef AnimPoint3_t vertex_t;

typedef struct {
	float v[3][3];
	float t[3];
} matrix_t;

typedef struct {
	point_t		min;
	point_t		max;
} bbox_t;

typedef struct {
	int type;
	vector_t normal;
	float dist;
} plane_t;

typedef AnimQuat_t quat_t;
typedef AnimAngAxis_t angleaxis_t;

//===========================================================================================

#ifndef PI
#define PI 3.14159265358979323846264338327950288419716939937510f
#endif

extern BOOL rad;

#define MathLibCos(x)	cos(rad ? x : ((x)*PI/180.0))
#define MathLibSin(x)	sin(rad ? x : ((x)*PI/180.0))
#define MathLibTan(x)	tan(rad ? x : ((x)*PI/180.0))

//===========================================================================================

extern matrix_t			IdentityMatrix;
extern point_t			origin;

//===========================================================================================

void MatrixGenerateTranslation (matrix_t *m, vector_t *vec);
void MatrixGenerateScaling (matrix_t *m, vector_t *vec);
void MatrixGenerateXRotate (matrix_t *m, float r);
void MatrixGenerateYRotate (matrix_t *m, float r);
void MatrixGenerateZRotate (matrix_t *m, float r);
void MatrixGenerateRotation (matrix_t *m, float x, float y, float z);
void MatrixGenerateVectorRotation (matrix_t *m, vector_t *vec, float a);
void MatrixMultiply (matrix_t *m1, matrix_t *m2, matrix_t *out);
void MatrixApply (matrix_t *m, point_t *p_in, point_t *p_out);
void MatrixFromAnimMatrix3 (AnimMatrix3_t *in, matrix_t *out);

float VectorDotP (vector_t *vec1, vector_t *vec2);
void VectorCrossP (vector_t *vec1, vector_t *vec2, vector_t *out);
float VectorLenght (vector_t *vec);
void VectorNormalize (vector_t *vec);
void VectorCopy (vector_t *src, vector_t *dest);
void VectorAdd (vector_t *vec0, vector_t *vec1, vector_t *dest);
void VectorSubtract (vector_t *vec0, vector_t *vec1, vector_t *dest);
void VectorMultByScalar (vector_t *vec, float scalar, vector_t *dest);
int VectorType (vector_t *v);

void QuatMul(quat_t *q1, quat_t *q2, quat_t *res);
void QuatSlerp(quat_t *from, quat_t *to, float t, quat_t *res);
void QuatToMatrix(quat_t *quat, matrix_t *m);
void QuatToAngleAxis(quat_t *quat, angleaxis_t *angleaxis);
void QuatFromEuler(float roll, float pitch, float yaw, quat_t *quat);

void PlaneSetType (plane_t *p);
void PlaneInvertNormal (plane_t *p);

//===========================================================================================

#endif